package com.example.pibd22_kalyshev_y_v_motorboat_hard;

import javafx.scene.canvas.GraphicsContext;
import java.util.Random;

public abstract class AbstractMap {
    private IDrawningObject _drawningObject = null;
    private GraphicsContext _graphicsContext = null;
    protected int[][] _map = null;
    protected int _width;
    protected int _height;
    protected float _size_x;
    protected float _size_y;
    protected final Random _random = new Random();
    protected final int _freeRoad = 0;
    protected final int _barrier = 1;
    public  void CreateMap(int width, int height, IDrawningObject drawningObject, GraphicsContext gc) {
        _width = width;
        _height = height;
        _drawningObject = drawningObject;
        _graphicsContext = gc;
        GenerateMap();
        while (!SetObjectOnMap()) {
            GenerateMap();
        }
        DrawMapWithObject();
    }
    public void MoveObject(Direction direction) {
        boolean enoughPlace = false;
        switch (direction) {
            case Up:
                enoughPlace = CheckEnoughPlace(0, (_drawningObject.GetStep() + 30) * -1);
                break;
            case Down:
                enoughPlace = CheckEnoughPlace(0, _drawningObject.GetStep());
                break;
            case Left:
                enoughPlace = CheckEnoughPlace(_drawningObject.GetStep() * -1, 0);
                break;
            case Right:
                enoughPlace = CheckEnoughPlace(_drawningObject.GetStep(), 0);
                break;
        }
        if (enoughPlace) {
            _drawningObject.MoveObject(direction);
        }
        DrawMapWithObject();
    }
    private boolean SetObjectOnMap() {
        if (_drawningObject == null || _map == null) {
            return false;
        }
        int x = _random.nextInt(10);
        int y = _random.nextInt(50);
        _drawningObject.SetObject(x, y, _width, _height);
        while (!CheckEnoughPlace(0, -40)) {
            x += 10;
            if (x >= _width) {
                if (y <= _height) {
                    y += 10;
                    x = 0;
                } else {
                    return false;
                }
            }
            _drawningObject.SetObject(x, y, _width, _height);
        }
        return true;
    }
    private boolean CheckEnoughPlace(float x, float y) {
        float[] position = _drawningObject.GetCurrentPosition();
        float right = (position[2] + x) / _size_x > 0 ? (position[2] + x) / _size_x : 0;
        float left = (position[0] + x) / _size_x > 0 ? (position[0] + x) / _size_x : 0;
        float up = (position[1] + y) / _size_y > 0 ? (position[1] + y) / _size_y : 0;
        float down = (position[3] + y) / _size_y > 0 ? (position[3] + y) / _size_y : 0;

        if (position[3] + y > _height || position[2] + x > _width || position[1] + y < 0 || position[0] + x < 0) return false;
        for (float i = left; i <= right; i+=0.5) {
            for (float j = up; j <= down; j+=0.5) {
                if (_map[Math.round(i)][Math.round(j)] == _barrier) {
                    return false;
                }
            }
        }
        return true;
    }

    public void DrawMapWithObject() {
        if (_drawningObject == null || _map == null) {
            return;
        }

        for (int i = 0; i < _map.length; i++) {
            for (int j = 0; j < _map[i].length; j++) {
                if (_map[i][j] == _freeRoad) {
                    DrawRoadPart(_graphicsContext, i, j);
                }
                else if (_map[i][j] == _barrier) {
                    DrawBarrierPart(_graphicsContext, i, j);
                }
            }
        }
        _drawningObject.DrawningObject(_graphicsContext);
    }

    protected abstract void GenerateMap();
    protected abstract void DrawRoadPart(GraphicsContext gc, int i, int j);
    protected abstract void DrawBarrierPart(GraphicsContext gc, int i, int j);
}
