package com.example.pibd22_kalyshev_y_v_motorboat_hard;

import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;

import java.util.Random;

public class ControllerMap {
    @FXML
    private Pane root;
    @FXML
    private Canvas canvas;
    @FXML
    private FlowPane flowPane;
    @FXML
    private Label labelSpeedValue;
    @FXML
    private Label labelWeightValue;
    @FXML
    private Label labelBodyColorValue;
    @FXML
    private ComboBox<String> comboBoxNumOars;
    @FXML
    private ComboBox<String> comboBoxOarsType;
    @FXML
    private ComboBox<String> comboBoxSelectorMap;
    @FXML
    private Button buttonCreate;
    @FXML
    private Button buttonCreateModif;
    @FXML
    private Button buttonLeft;
    @FXML
    private Button buttonRight;
    @FXML
    private Button buttonUp;
    @FXML
    private Button buttonDown;
    private final double buttonMargin = 10.0;
    private AbstractMap _abstractMap;
    @FXML
    public void initialize() {
        buttonCreate.setTranslateX(buttonMargin);

        root.widthProperty().addListener((obs, oldVal, newVal) -> {
            UpdateGUI();
            if (_abstractMap != null) {
                _abstractMap.DrawMapWithObject();
            }
        });
        root.heightProperty().addListener((obs, oldVal, newVal) -> {
            UpdateGUI();
            if (_abstractMap != null) {
                _abstractMap.DrawMapWithObject();
            }
        });
    }
    @FXML
    void ButtonCreate_Click() {
        Random rnd = new Random();
        DrawningBoat boat = new DrawningBoat(rnd.nextInt(200) + 100, rnd.nextInt(1000) + 1000,
                Color.rgb(rnd.nextInt(256), rnd.nextInt(256), rnd.nextInt(256)));
        SetData(boat);
    }
    @FXML
    void ButtonCreateModif_Click() {
        Random rnd = new Random();
        DrawningSpeedboat boat = new DrawningSpeedboat(rnd.nextInt(200) + 100, rnd.nextInt(1000) + 1000,
                Color.rgb(rnd.nextInt(256), rnd.nextInt(256), rnd.nextInt(256)),
                Color.rgb(rnd.nextInt(256), rnd.nextInt(256), rnd.nextInt(256)),
                (rnd.nextInt(2) != 0), (rnd.nextInt(2) != 0), (rnd.nextInt(2) != 0));
        SetData(boat);
    }
    @FXML
    void ButtonMove_Click(ActionEvent event) {
        if (_abstractMap == null) {
            return;
        }
        String buttonName = ((Button) event.getSource()).getId();
        switch (buttonName) {
            case "buttonUp" -> _abstractMap.MoveObject(Direction.Up);
            case "buttonDown" -> _abstractMap.MoveObject(Direction.Down);
            case "buttonLeft" -> _abstractMap.MoveObject(Direction.Left);
            case "buttonRight" -> _abstractMap.MoveObject(Direction.Right);
        }
    }
    @FXML
    private void ComboBoxSelectorMap_Changed() {
        switch (comboBoxSelectorMap.getValue()) {
            case "Simple map" -> _abstractMap = new SimpleMap();
            case "My second map" -> _abstractMap = new MySecondMap();
        }
    }
    private void ChangeDrawningOars(DrawningBoat boat) {
        IDrawningAdditionalElement newDrawningOars = switch (comboBoxOarsType.getValue()) {
                    case "None" -> new DrawningOars(boat.GetBoat().GetBodyColor());
                    case "Slim" -> new DrawningSlimOars(boat.GetBoat().GetBodyColor());
                    case "Large" -> new DrawningLargeOars(boat.GetBoat().GetBodyColor());
                    default -> null;
                };
        if (newDrawningOars != null) {
            boat.SetDrawningOars(newDrawningOars);
            boat.GetDrawningOars().SetNumberOars(Integer.parseInt(comboBoxNumOars.getValue()));
        }
    }
    private void UpdateGUI() {
        double sceneWidth = root.getWidth();
        double sceneHeight = root.getHeight();

        double flowPaneHeight = flowPane.getHeight();
        double buttonCreateHeight = buttonCreate.getHeight();

        canvas.setWidth(sceneWidth);
        flowPane.setPrefWidth(sceneWidth);
        canvas.setHeight(sceneHeight - flowPaneHeight);
        flowPane.setTranslateY(sceneHeight - flowPaneHeight);

        buttonCreate.setTranslateY(sceneHeight - flowPaneHeight - buttonCreateHeight - buttonMargin);
        buttonCreateModif.setTranslateY(sceneHeight - flowPaneHeight - buttonCreateHeight - buttonMargin);

        int buttonMoveHeight = 30;
        int distanceBetweenMoveButtons = 5;
        buttonUp.setTranslateY(sceneHeight - flowPaneHeight - buttonMoveHeight * 2.0 - buttonMargin - distanceBetweenMoveButtons);
        int buttonMoveWidth = 30;
        buttonUp.setTranslateX(sceneWidth - buttonMargin - buttonMoveWidth * 2.0 - distanceBetweenMoveButtons);

        buttonDown.setTranslateY(sceneHeight - flowPaneHeight - buttonMoveHeight - buttonMargin);
        buttonDown.setTranslateX(sceneWidth- buttonMargin - buttonMoveWidth * 2.0 - distanceBetweenMoveButtons);

        buttonLeft.setTranslateY(sceneHeight - flowPaneHeight - buttonMoveHeight - buttonMargin);
        buttonLeft.setTranslateX(sceneWidth - buttonMargin - buttonMoveWidth * 3.0 - distanceBetweenMoveButtons * 2.0);

        buttonRight.setTranslateY(sceneHeight - flowPaneHeight - buttonMoveHeight - buttonMargin);
        buttonRight.setTranslateX(sceneWidth - buttonMargin - buttonMoveWidth);
    }
    private void SetData(DrawningBoat boat) {
        ChangeDrawningOars(boat);
        labelSpeedValue.setText(Integer.toString(boat.GetBoat().GetSpeed()));
        labelWeightValue.setText(Double.toString(boat.GetBoat().GetWeight()));
        labelBodyColorValue.setText(boat.GetBoat().GetBodyColor().toString());
        GraphicsContext gc = canvas.getGraphicsContext2D();
        gc.clearRect(0.0, 0.0, canvas.getWidth(), canvas.getHeight());
        _abstractMap.CreateMap((int)canvas.getWidth(), (int)canvas.getHeight(), new DrawningObjectBoat(boat), gc);
    }
}
