package com.example.pibd22_kalyshev_y_v_motorboat_hard;

import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;

public class DrawningBoat {
    private EntityBoat _boat;
    private IDrawningAdditionalElement _drawningOars;
    protected float _startPosX;
    protected float _startPosY;
    private Integer _pictureWidth;
    private Integer _pictureHeight;
    private int _boatWidth = 170;
    private int _boatHeight = 60;
    public EntityBoat GetBoat()
    {
        return _boat;
    }
    protected void SetBoat(EntityBoat boat)
    {
        _boat = boat;
    }
    public IDrawningAdditionalElement GetDrawningOars() {
        return _drawningOars;
    }
    public void SetDrawningOars(IDrawningAdditionalElement drawningOars)
    {
        _drawningOars = drawningOars;
    }
    public DrawningBoat(int speed, float weight, Color bodyColor) {
        _boat = new EntityBoat(speed, weight, bodyColor);
        _drawningOars = new DrawningOars(bodyColor);
    }
    protected DrawningBoat(int speed, float weight, Color bodyColor, int boatWidth, int boatHeight) {
        this(speed, weight, bodyColor);
        _boatHeight = boatHeight;
        _boatWidth = boatWidth;
    }
    public void SetPosition(int x, int y, int width, int height) {
        if (x < 0 || y < 0) {
            return;
        }
        if (x + _boatWidth > width || y + _drawningOars.deltaYdown + _boatHeight > height) {
            return;
        }
        _startPosX = x;
        _startPosY = y;
        _pictureWidth = width;
        _pictureHeight = height;
    }
    public void MoveTransport(Direction direction) {
        if (_pictureWidth == null || _pictureHeight == null) {
            return;
        }
        switch (direction) {
            case Up:
                if (_startPosY - _boat.GetStep() - _drawningOars.deltaYup > 0) {
                    _startPosY -= _boat.GetStep();
                }
                break;
            case Down:
                if (_startPosY + _boatHeight + _boat.GetStep() + _drawningOars.deltaYdown < _pictureHeight) {
                    _startPosY += _boat.GetStep();
                }
                break;
            case Left:
                if (_startPosX - _boat.GetStep() > 0) {
                    _startPosX -= _boat.GetStep();
                }
                break;
            case Right:
                if (_startPosX + _boatWidth + _boat.GetStep() < _pictureWidth) {
                    _startPosX += _boat.GetStep();
                }
                break;
            default:
                break;
        }
    }
    public void DrawTransport(GraphicsContext gc) {
        if (_startPosX < 0 || _startPosY < 0 || _pictureWidth == null || _pictureHeight == null) {
            return;
        }
        _startPosY += 40;
        // Корпус
        gc.setStroke(Color.BLACK);
        gc.setLineWidth(2);
        gc.setFill(_boat.GetBodyColor());
        gc.fillPolygon(new double[] {_startPosX, _startPosX+120, _startPosX+170, _startPosX+120, _startPosX, _startPosX},
                new double[] {_startPosY, _startPosY, _startPosY+30, _startPosY+60, _startPosY+60, _startPosY}, 6);
        gc.strokePolyline(new double[] {_startPosX, _startPosX+120, _startPosX+170, _startPosX+120, _startPosX, _startPosX},
                new double[] {_startPosY, _startPosY, _startPosY+30, _startPosY+60, _startPosY+60, _startPosY}, 6);
        // Седло
        gc.setFill(Color.BROWN);
        gc.fillOval(_startPosX+10, _startPosY+10, 110, 40);
        _startPosY -= 40;
        _drawningOars.DrawOars(gc, _startPosX, _startPosY);

    }
    public void ChangeBorders(int width, int height) {
        _pictureWidth = width;
        _pictureHeight = height;
        if (_pictureWidth <= _boatWidth || _pictureHeight <= _boatHeight) {
            _pictureWidth = null;
            _pictureHeight = null;
            return;
        }
        if (_startPosX + _boatWidth > _pictureWidth) {
            _startPosX = _pictureWidth - _boatWidth;
        }
        if (_startPosY + _boatHeight > _pictureHeight) {
            _startPosY = _pictureHeight - _boatHeight;
        }
    }
    public float[] GetCurrentPosition() {
        float[] position = new float[4];
        position[0] = _startPosX;
        position[1] = _startPosY;
        position[2] = _startPosX + _boatWidth;
        position[3] = _startPosY + _boatHeight;
        return position;
    }
}
